# Copyright (C) 2015 Tawfik Moher Alsady, Florian Beissner
#
# beissner.florian@mh-hannover.de
#
# Developed at
# Somatosensory and Autonomic Therapy Research,
# Institute for Neuroradiology, Hannover Medical School,
# Hannover, Germany
#
# This file is part of the mICA toolbox.
#
# The mICA toolbox is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The mICA toolbox is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the mICA toolbox.  If not, see <http://www.gnu.org/licenses/>. 

#requires: clue
library("clue")


args<-commandArgs(TRUE)

if (length(args) < 3) {
  stop ("Usage: ic_corr.r in_prefix samples dims")
}

main_folder = args[1]

trialscount = as.numeric(args[2])
if ( trialscount > 0 ) {
  trials = c(1000:(trialscount+999))
}else{
  trials = c(1000)
}

dimsstr = args[3]
dimsstr_split = strsplit(dimsstr, ",")[[1]]
dims=c()
for (dim in dimsstr_split) {
  if( grepl("-", dim) == TRUE) {
    minmax = strsplit(dim, "-")[[1]]
    dim=c(minmax[1]:minmax[2])
  }
  dims=c(dims, dim)
}


cor_means_mat = matrix(, nrow = length(trials), ncol = length(dims))

colnames(cor_means_mat) = paste( "dim_", dims, sep="")
rownames(cor_means_mat) = paste( "trial_", trials, sep="")

prefix_fsl = paste ("$FSLDIR/bin/fslcc -t -1 -p 5 -m ", " ", sep="")


for(trialnum in trials) {
  
  for (dim in dims){
    
    g1_mask = paste(main_folder, "/sample_", trialnum ,"/group1/dim", dim, "/mask.nii.gz", sep="")
    
    ICs_g1 = paste(main_folder, "/sample_", trialnum ,"/group1/dim", dim, "/melodic_IC.nii.gz", sep="")
    ICs_g2 = paste(main_folder, "/sample_", trialnum ,"/group2/dim", dim, "/melodic_IC.nii.gz", sep="")
    
    out_txt = paste(main_folder, "/sample_", trialnum , "/corr_dim", dim, ".txt", sep="")
    
    system(paste(prefix_fsl, g1_mask, " ", ICs_g1, " ", ICs_g2, " > ", out_txt ,sep=""))
    
    cor_vec = try( read.table(out_txt, quote="\""), silent = TRUE )
    if(class(cor_vec) != "try-error") {
      
      table_dim=length(unique(cor_vec[,1]))
      cor_mat = matrix(cor_vec[,3], nrow = table_dim, ncol = table_dim )
      cor_mat=abs(cor_mat)
      
      y = solve_LSAP(cor_mat, maximum = TRUE)
      
      cor_means_mat[ paste( "trial_", trialnum, sep="") , paste( "dim_", dim, sep="") ] =  mean(cor_mat[cbind(seq_along(y), y)])
      
    } else {
      cor_means_mat[ paste( "trial_", trialnum, sep="") , paste( "dim_", dim, sep="") ] = NA;
      
    }
  }
  
}

write.csv(cor_means_mat, file=paste(main_folder, "/corr_means.csv",  sep="") )

# cor_means_mat
plotwidth = 100 + (15 * length(dims))
if (plotwidth < 480) {
  plotwidth = 480
}else if (plotwidth > 900) {
  plotwidth = 900
}

png(filename=paste(main_folder, "/corr_mean.png",  sep=""),
    width = plotwidth, height = 480)

plot (colMeans(cor_means_mat, na.rm = TRUE),, xaxt="n", type='o', xlab='Dimensionality', ylab='Pearsons correaltion')
#hist(off_diagonal_mat, breaks = 40)
axis(1, at=1:length(dims), labels=dims)
title('Mean correlation of matched IC-pairs (Reproducibility)')

dev.off()

print (cor_means_mat)

png(filename=paste(main_folder, "/no_convergence_error.png",  sep=""),
    width = plotwidth, height = 480)
plot (colSums(is.na(cor_means_mat)),, xaxt="n", type='o', xlab='Dimensionality', ylab='Frequency of no convergence / Error')
axis(1, at=1:length(dims), labels=dims)
title('Frequency of ICA No convergence or Error')
dev.off()

