# Copyright (C) 2015 Tawfik Moher Alsady, Florian Beissner
#
# beissner.florian@mh-hannover.de
#
# Developed at
# Somatosensory and Autonomic Therapy Research,
# Institute for Neuroradiology, Hannover Medical School,
# Hannover, Germany
#
# This file is part of the mICA toolbox.
#
# The mICA toolbox is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The mICA toolbox is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with the mICA toolbox.  If not, see <http://www.gnu.org/licenses/>. 


args<-commandArgs(TRUE)

if (length(args) < 3) {
  stop ("Usage: splithalf list_filename permutations out_prefix")
}


infile = args[1];
permu = as.numeric(args[2]);
outprefix = args[3];

if (permu < 0) {
  stop ("Error: permutations number is not valid")
}else if (permu > 0) {
  trials = seq(1000, 1000 + (permu -1) )
}else{
  trials = c(1000)  
}


fileslist=t(read.table(infile, quote=""))

if (length(fileslist) < 2) {
  stop ("Error: files list is not valid")
}

groupsize=floor(length(fileslist)/2)


for(trialnum in trials) {
  
  output0 = paste(outprefix, "/sample_", trialnum, sep="")
  dir.create( output0 )

  
  if (permu != 0) {
    rand_allsubj = sample(fileslist);
  }else{
    rand_allsubj = fileslist;
  }

  for (groupnum in 0:1){

    output1 = paste(output0, "/group", (groupnum+1), sep="")
    dir.create( output1 )

    write(rand_allsubj[(1+(groupnum*groupsize)):(groupsize+(groupnum*groupsize))],
      file = paste(output1, "_input.txt", sep =""),
      sep = "\n")
  
  }

}
